package ru.ttyh.neko259.notey.db;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.util.Utility;

import java.util.ArrayList;
import java.util.List;

public class NoteDAO {
	
	public static final String TABLE_NOTES = "notes";
	
	public static final String COLUMN_ID = "id";
	public static final String COLUMN_TITLE = "title";
	public static final String COLUMN_TEXT = "text";
	public static final String COLUMN_DATE = "date";
    public static final String COLUMN_LATITUDE = "latitude";
    public static final String COLUMN_LONGITUDE = "longitude";

    private static final String NOTE_ID_SELECTION = COLUMN_ID + " = ?";
	private static final String NOTE_SEARCH = "(LOWER(" + COLUMN_TITLE + ") " +
            "LIKE ?) OR (LOWER(" + COLUMN_TEXT + ") LIKE ?)";
	private static final String NOTES_ORDER = COLUMN_DATE + " DESC";
	
	private SQLiteDatabase db;
	
	public NoteDAO(SQLiteDatabase db) {
		this.db = db;
	}
	
	public List<NoteBean> getNotes() {
		List<NoteBean> notes = new ArrayList<NoteBean>();
		
		String[] columns = new String[] {
				COLUMN_ID,
				COLUMN_TITLE,
				COLUMN_TEXT,
				COLUMN_DATE,
                COLUMN_LATITUDE,
                COLUMN_LONGITUDE
		};
		
		Cursor cur = db.query(TABLE_NOTES, columns, null, null, null, null,
				NOTES_ORDER);
		while (cur.moveToNext()) {
			NoteBean note = new NoteBean();
			note.setId(cur.getLong(cur.getColumnIndex(COLUMN_ID)));
			note.setTitle(cur.getString(cur.getColumnIndex(COLUMN_TITLE)));
			note.setText(cur.getString(cur.getColumnIndex(COLUMN_TEXT)));
			String dateText = cur.getString(cur.getColumnIndex(COLUMN_DATE));

            note.setLatitude(cur.getDouble(cur.getColumnIndex(COLUMN_LATITUDE)));
            note.setLongitude(cur.getDouble(cur.getColumnIndex(COLUMN_LONGITUDE)));

			note.setDate(Utility.getDate(dateText));
			
			notes.add(note);
		}
		cur.close();
		
		return notes;
	}
	
	public List<NoteBean> findNotesByKey(String key) {
		List<NoteBean> notes = new ArrayList<NoteBean>();
		
		String[] columns = new String[] {
				COLUMN_ID,
				COLUMN_TITLE,
				COLUMN_TEXT,
				COLUMN_DATE,
                COLUMN_LATITUDE,
                COLUMN_LONGITUDE
		};
		String[] selectionArgs = new String[] {
				"%" + key.toLowerCase() + "%",
				"%" + key.toLowerCase() + "%"
		};
		
		Cursor cur = db.query(TABLE_NOTES, columns, NOTE_SEARCH, selectionArgs,
				null, null, NOTES_ORDER);
		while (cur.moveToNext()) {
			NoteBean note = new NoteBean();
			note.setId(cur.getLong(cur.getColumnIndex(COLUMN_ID)));
			note.setTitle(cur.getString(cur.getColumnIndex(COLUMN_TITLE)));
			note.setText(cur.getString(cur.getColumnIndex(COLUMN_TEXT)));
			String dateText = cur.getString(cur.getColumnIndex(COLUMN_DATE));
			
			note.setDate(Utility.getDate(dateText));

            note.setLatitude(cur.getDouble(cur.getColumnIndex(COLUMN_LATITUDE)));
            note.setLongitude(cur.getDouble(cur.getColumnIndex(COLUMN_LONGITUDE)));

			notes.add(note);
		}
		cur.close();
		
		return notes;
	}
	
	public NoteBean getNoteById(long id) {
		NoteBean note = null;
		
		String[] columns = new String[] {
				COLUMN_TITLE,
				COLUMN_TEXT,
				COLUMN_DATE,
                COLUMN_LATITUDE,
                COLUMN_LONGITUDE
		};
		String[] selectionArgs = new String[] {
				Long.toString(id)
		};
		
		Cursor cur = db.query(TABLE_NOTES, columns, NOTE_ID_SELECTION,
				selectionArgs, null, null, null);
		if (cur.moveToFirst()) {
			note = new NoteBean();
			
			note.setId(id);
			note.setTitle(cur.getString(cur.getColumnIndex(COLUMN_TITLE)));
			note.setText(cur.getString(cur.getColumnIndex(COLUMN_TEXT)));
			String dateText = cur.getString(cur.getColumnIndex(COLUMN_DATE));
			
			note.setDate(Utility.getDate(dateText));

            note.setLatitude(cur.getDouble(cur.getColumnIndex(COLUMN_LATITUDE)));
            note.setLongitude(cur.getDouble(cur.getColumnIndex(COLUMN_LONGITUDE)));
        }
		
		cur.close();
		
		return note;
	}
	
	public void save(NoteBean note) {
		ContentValues values = new ContentValues();
		values.put(COLUMN_TITLE, note.getTitle());
		values.put(COLUMN_TEXT, note.getText());
		values.put(COLUMN_DATE, Utility.getDateString(note.getDate()));
        values.put(COLUMN_LATITUDE, note.getLatitude());
        values.put(COLUMN_LONGITUDE, note.getLongitude());
		
		db.insert(TABLE_NOTES, null, values);
	}
	
	public void update(long id, NoteBean newNote) {
		ContentValues values = new ContentValues();
		values.put(COLUMN_TITLE, newNote.getTitle());
		values.put(COLUMN_TEXT, newNote.getText());
		values.put(COLUMN_DATE, Utility.getDateString(newNote.getDate()));
        values.put(COLUMN_LATITUDE, newNote.getLatitude());
        values.put(COLUMN_LONGITUDE, newNote.getLongitude());

		String[] selectionArgs = new String[] {
				Long.toString(id)
		};
		
		db.update(TABLE_NOTES, values, NOTE_ID_SELECTION, selectionArgs);
	}
	
	public void delete(long id) {
		String[] selectionArgs = new String[] {
				Long.toString(id)
		};

		db.delete(TABLE_NOTES, NOTE_ID_SELECTION, selectionArgs);
	}

}