package ru.ttyh.neko259.notey.db;

import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.SparseArray;
import ru.ttyh.neko259.notey.app.NotesApplication;

import java.util.ArrayList;
import java.util.List;

public class NotesDBOpenHelper extends SQLiteOpenHelper {

    private static final int VERSION_INITIAL = 1;
    private static final int VERSION_NOTE_LOCATION = 2;
	
	private static final String DB_NAME = "notes";
    private static final int DB_VERSION = VERSION_NOTE_LOCATION;

    private static final SparseArray<List<String>> updateScripts =
            new SparseArray<List<String>>();

    private static final String CREATE_TABLE_NOTES = "CREATE TABLE "
			+ NoteDAO.TABLE_NOTES + " (" + NoteDAO.COLUMN_ID
			+ " INTEGER PRIMARY KEY AUTOINCREMENT, " + NoteDAO.COLUMN_TITLE
			+ " TEXT, " + NoteDAO.COLUMN_TEXT + " TEXT, " + NoteDAO.COLUMN_DATE
			+ " TEXT);";

    static {
        List<String> secondVersionUpdate = new ArrayList<String>();
        secondVersionUpdate.add("ALTER TABLE " + NoteDAO
                .TABLE_NOTES + " ADD " + NoteDAO.COLUMN_LATITUDE + " REAL");
        secondVersionUpdate.add("ALTER TABLE " + NoteDAO
                .TABLE_NOTES + " ADD " + NoteDAO.COLUMN_LONGITUDE + " REAL");

        updateScripts.put(VERSION_NOTE_LOCATION, secondVersionUpdate);
    }

    public NotesDBOpenHelper() {
		super(NotesApplication.getContext(), DB_NAME, null, DB_VERSION);
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL(CREATE_TABLE_NOTES);

        updateDb(db, 1);
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        updateDb(db, oldVersion);
    }

    private void updateDb(SQLiteDatabase db, int fromVersion) {
        for (int i = fromVersion + 1; i <= DB_VERSION; i++) {
            for (String script : updateScripts.get(i)) {
                if (script != null) {
                    db.execSQL(script);
                }
            }
        }
    }

}