package ru.ttyh.neko259.notey.ui;

import android.app.Activity;
import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.EditText;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.DBManager;
import ru.ttyh.neko259.notey.db.NoteDAO;
import ru.ttyh.neko259.notey.ui.adapters.TagsListAdapter;
import ru.ttyh.neko259.notey.ui.ctrl.HorizontalListView;
import ru.ttyh.neko259.notey.util.Constants;
import ru.ttyh.neko259.notey.util.Parser;

import java.util.Date;

public class AddNoteActivity extends Activity implements AdapterView
        .OnItemClickListener {

    public static final int NO_NOTE = -1;
    public static final int LOCATION_UPDATE_TIME = 30;
    public static final int LOCATION_UPDATE_DISTANCE = 10;

    private long noteId;
    TagsListAdapter adapter;

    private Location currentLocation;
    private LocationListener locationListener;

    @Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.add_note_activity);
		
		EditText noteTitle = (EditText) findViewById(R.id.note_title_edit);
		EditText noteText = (EditText) findViewById(R.id.note_text_edit);

		NoteDAO noteDAO = DBManager.getInstance().getNoteDAO();

		noteId = getIntent().getLongExtra(Constants.EXTRA_NOTE_ID, NO_NOTE);
		if (noteId > NO_NOTE) {
			NoteBean note = noteDAO.getNoteById(noteId);
			noteTitle.setText(note.getTitle());
			noteText.setText(note.getText());
		}

        initList();
        initLocationListener();
	}

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.edit, menu);

        return true;
    }

    @Override
    public boolean onMenuItemSelected(int featureId, MenuItem item) {
        if (R.id.menu_save == item.getItemId()) {
            saveNote();

            finish();
        }

        return super.onMenuItemSelected(featureId, item);
    }

    private void saveNote() {
        EditText noteTitle = (EditText) findViewById(R.id.note_title_edit);
        EditText noteText = (EditText) findViewById(R.id.note_text_edit);

        NoteBean note = new NoteBean();
        note.setTitle(noteTitle.getText().toString());
        note.setText(noteText.getText().toString());
        note.setDate(new Date());

        if (currentLocation != null) {
            note.setLatitude(currentLocation.getLatitude());
            note.setLongitude(currentLocation.getLongitude());
        }

        NoteDAO noteDAO = DBManager.getInstance().getNoteDAO();
        if (noteId == NO_NOTE) {
            noteDAO.save(note);
        } else {
            noteDAO.update(noteId, note);
        }
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position,
                            long id) {
        String tag = (String) adapter.getItem(position);
        tag = Parser.getTagName(tag);

        EditText noteText = (EditText) findViewById(R.id.note_text_edit);
        String oldText = noteText.getText().toString();
        int cursorPosition = noteText.getSelectionStart();
        noteText.setText(oldText.substring(0, cursorPosition) + tag + oldText
                .substring(cursorPosition));
        noteText.setSelection(cursorPosition + tag.length());
    }

    @Override
    protected void onStop() {
        super.onStop();

        LocationManager manager = (LocationManager) getSystemService(
                Context.LOCATION_SERVICE);
        manager.removeUpdates(locationListener);
    }

    private void initList() {
        HorizontalListView list = (HorizontalListView) findViewById(R.id
                .tags_list);

        adapter = new TagsListAdapter(this);
        list.setAdapter(adapter);
        list.setOnItemClickListener(this);
    }

    private void initLocationListener() {
        LocationManager manager = (LocationManager) getSystemService(
                Context.LOCATION_SERVICE);

        if (locationListener == null) {
            locationListener = new LocationUpdateListener();

            currentLocation = manager.getLastKnownLocation(LocationManager
                    .GPS_PROVIDER);
        }

        manager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
                LOCATION_UPDATE_TIME, LOCATION_UPDATE_DISTANCE,
                locationListener);
        manager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
                LOCATION_UPDATE_TIME, LOCATION_UPDATE_DISTANCE,
                locationListener);
    }

    private class LocationUpdateListener implements LocationListener {

        @Override
        public void onLocationChanged(Location location) {
            currentLocation = location;
        }

        @Override
        public void onStatusChanged(String provider, int status, Bundle extras) {}

        @Override
        public void onProviderEnabled(String provider) {}

        @Override
        public void onProviderDisabled(String provider) {}

    }

}