package ru.ttyh.neko259.notey.ui;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.DBManager;
import ru.ttyh.neko259.notey.db.NoteDAO;
import ru.ttyh.neko259.notey.ui.adapters.NotesAdapter;
import ru.ttyh.neko259.notey.ui.dialog.ImportConfirmDialog;
import ru.ttyh.neko259.notey.util.Constants;
import ru.ttyh.neko259.notey.util.export.Exporter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class NotesActivity extends FragmentActivity {

    private static final String IMPORT_DIALOG = "importDialog";

    private String key;
    private NotesAdapter adapter;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.notes);

        key = getIntent().getStringExtra(Constants.EXTRA_KEY);

        if (key != null) {
            TextView tagView = (TextView) findViewById(R.id.tag);
            tagView.setText(key);
            tagView.setVisibility(View.VISIBLE);
        }
    }
    
    @Override
    protected void onResume() {
    	super.onResume();
    	
        ListView noteList = (ListView) findViewById(R.id.note_list);

        List<NoteBean> notes = getNotes();

        adapter = new NotesAdapter(notes, this);
        noteList.setAdapter(adapter);
        noteList.setOnItemClickListener(adapter);
        noteList.setOnItemLongClickListener(adapter);
    }

    @Override
    public boolean onSearchRequested() {
    	if (key == null) {
    		AlertDialog.Builder builder = new AlertDialog.Builder(this);
    		builder.setTitle(R.string.search);

    		final View view = getLayoutInflater().inflate(R.layout.search_dialog,
    				null);
    		builder.setView(view);

    		builder.setNegativeButton(android.R.string.cancel, null);
    		builder.setPositiveButton(android.R.string.ok, new OnClickListener() {

    			public void onClick(DialogInterface dialog, int which) {
    				Intent intent = new Intent(NotesActivity.this, NotesActivity.class);
    				String newKey = ((EditText) view.findViewById(
    						R.id.search_key)).getText().toString();
    				intent.putExtra(Constants.EXTRA_KEY, newKey);
    				NotesActivity.this.startActivity(intent);
    			}

    		});
    		builder.show();
    		return true;
    	}

    	return false;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.notes, menu);

        return true;
    }

    @Override
    public boolean onMenuItemSelected(int featureId, MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_help:
                openHelp();
                break;

            case R.id.menu_add:
                openAdd();
                break;

            case R.id.menu_export:
                exportAll();
                break;

            case R.id.menu_import:
                ImportConfirmDialog dialog = new ImportConfirmDialog();
                dialog.setOnImportCompletedListener(new ImportConfirmDialog.OnImportCompletedListener() {
                    @Override
                    public void onCompleted() {
                        adapter.updateItems(DBManager.getInstance()
                                .getNoteDAO().getNotes());
                    }
                });
                dialog.show(getSupportFragmentManager(), IMPORT_DIALOG);
                break;

            case R.id.menu_search:
                onSearchRequested();
                break;

            case R.id.menu_map:
                openMap();
                break;
        }

        return true;
    }

    private List<NoteBean> getNotes() {
        List<NoteBean> notes = new ArrayList<NoteBean>();

        NoteDAO noteDAO = DBManager.getInstance().getNoteDAO();
        if (key == null) {
            notes.addAll(noteDAO.getNotes());
        } else {
            notes.addAll(noteDAO.findNotesByKey(key));
        }
        return notes;
    }

    private void openHelp() {
        Intent intent = new Intent(this, HelpActivity.class);
        startActivity(intent);
    }

    private void openAdd() {
        Intent intent = new Intent(this, AddNoteActivity.class);
        startActivity(intent);
    }

    private void exportAll() {
        new ExportTask().execute();
    }

    private void openMap() {
        Intent intent = new Intent(this, MapActivity.class);
        startActivity(intent);
    }

    private class ExportTask extends AsyncTask<Void, Void, Boolean> {

        @Override
        protected Boolean doInBackground(Void... params) {
            List<NoteBean> notes = DBManager.getInstance().getNoteDAO()
                    .getNotes();

            Exporter.clearExportDirectory();
            for (NoteBean note : notes) {
                try {
                    Exporter.exportNote(note.getId());
                } catch (IOException e) {
                    return false;
                }
            }

            return true;
        }

        @Override
        protected void onPostExecute(Boolean result) {
            int resultMessage = result ? R.string.done_export : R.string
                    .error_export;
            Toast.makeText(NotesActivity.this, resultMessage,
                    Toast.LENGTH_SHORT).show();
        }
    }


}