package ru.ttyh.neko259.notey.ui;

import android.app.Activity;
import android.content.Intent;
import android.location.Address;
import android.location.Geocoder;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Html;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.DBManager;
import ru.ttyh.neko259.notey.util.Constants;
import ru.ttyh.neko259.notey.util.Parser;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

public class ShowNoteActivity extends Activity {

    private static final String TITLE_START = "#";
	private static final String TITLE_DEL = " - ";
    private static final int NO_NOTE = -1;
    private static final String SHARE_TYPE = "text/html";

    private long noteId;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.show_note_activity);
		
		noteId = getIntent().getLongExtra(Constants.EXTRA_NOTE_ID, NO_NOTE);

        refreshText();
	}

    @Override
    protected void onResume() {
        super.onResume();

        refreshText();
    }

    private void refreshText() {
		NoteBean note = DBManager.getInstance().getNoteDAO().getNoteById(noteId);

		TextView noteTitle = (TextView) findViewById(R.id.note_title_text);
		noteTitle.setText(TITLE_START + Long.toString(noteId) + TITLE_DEL
				+ note.getTitle());

		TextView noteText = (TextView) findViewById(R.id.note_text_text);
		noteText.setText(Html.fromHtml(Parser.parseNoteText(note.getText())));

		TextView noteDate = (TextView) findViewById(R.id.note_date_text);
		Date date = note.getDate();
        String dateText = SimpleDateFormat.getDateTimeInstance().format(date);
		dateText = getResources().getString(R.string.last_edit) + " " +
                dateText;
		noteDate.setText(dateText);

		new LocationNameTask().execute(note);
	}

    private void showMap() {
        NoteBean note = DBManager.getInstance().getNoteDAO().getNoteById
                (noteId);

        if (NoteBean.NO_LOCATION != note.getLatitude() && NoteBean
                .NO_LOCATION != note.getLongitude()) {
            Intent intent = new Intent(ShowNoteActivity.this,
                    MapActivity.class);
            intent.putExtra(MapActivity.EXTRA_SINGLE_LOCATION, noteId);

            startActivity(intent);
        } else {
            // TODO Show a dialog to say "don't have a location"
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.show, menu);

        return true;
    }

    @Override
    public boolean onMenuItemSelected(int featureId, MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_edit:
                openEdit();
                break;

            case R.id.menu_share:
                openShare();
                break;

            case R.id.menu_map:
                showMap();
                break;
        }

        return true;
    }

    private void openEdit() {
        Intent intent = new Intent(this, AddNoteActivity.class);
        intent.putExtra(Constants.EXTRA_NOTE_ID, noteId);
        startActivity(intent);
    }

    private void openShare() {
        Intent sharingIntent = new Intent(android.content.Intent.ACTION_SEND);

        NoteBean note = DBManager.getInstance().getNoteDAO().getNoteById
                (noteId);

        sharingIntent.setType(SHARE_TYPE);
        sharingIntent.putExtra(android.content.Intent.EXTRA_SUBJECT,
                note.getTitle());
        sharingIntent.putExtra(android.content.Intent.EXTRA_TEXT,
                Parser.getHtmlNote(note.getText()));
        startActivity(sharingIntent);
    }

	private class LocationNameTask extends AsyncTask<NoteBean, Void, String> {

		@Override
		protected String doInBackground(NoteBean... params) {
			NoteBean note = params[0];

			double latitude = note.getLatitude();
			double longitude = note.getLongitude();
			String description = getResources().getString(
					R.string.no_location);
			if (NoteBean.NO_LOCATION != latitude && NoteBean.NO_LOCATION !=
					longitude) {
				Geocoder geocoder = new Geocoder(getApplicationContext(),
						Locale.getDefault());
				try {
					List<Address> listAddresses = geocoder.getFromLocation(
							note.getLatitude(), note.getLongitude(), 1);
					if (null != listAddresses && listAddresses.size() > 0) {
						String addressLine = "";
						description = "";

						for (int i = 0; addressLine != null; i++) {
							addressLine = listAddresses.get(0).getAddressLine(i);
							if (addressLine != null) {
								description += addressLine + " ";
							}
						}
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

			return description;
		}

		@Override
		protected void onPostExecute(String description) {
			TextView noteLocation = (TextView) findViewById(R.id
					.note_location_text);
			if (noteLocation != null) {
				noteLocation.setText(description);
			}
		}
	}
}