package ru.ttyh.neko259.notey.ui.adapters;

import android.content.Intent;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.TextView;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.ui.ShowNoteActivity;
import ru.ttyh.neko259.notey.ui.dialog.NoteActionsDialog;
import ru.ttyh.neko259.notey.ui.listener.OnListChangedListener;
import ru.ttyh.neko259.notey.util.Constants;
import ru.ttyh.neko259.notey.util.Utility;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class NotesAdapter extends BaseAdapter implements AdapterView
        .OnItemClickListener, AdapterView.OnItemLongClickListener {

    private static final String NOTE_ACTIONS_DIALOG = "noteActionsDialog";

    private List<NoteBean> notes;
	private FragmentActivity activity;
	
	public NotesAdapter(List<NoteBean> notes, FragmentActivity activity) {
		this.notes = notes;
		this.activity = activity;
	}

    public void updateItems(List<NoteBean> notes) {
        this.notes = notes;
        notifyDataSetChanged();
    }

	public int getCount() {
		return notes.size();
	}

	public Object getItem(int position) {
		return notes.get(position);
	}

	public long getItemId(int position) {
		return notes.get(position).getId();
	}

	public View getView(int position, View convertView, ViewGroup parent) {
		View view = convertView;
		if (view == null) {
			view = activity.getLayoutInflater().inflate(R.layout.note_list_item,
					parent, false);
		}
		
		final NoteBean note = notes.get(position);
		
		TextView title = (TextView) view.findViewById(R.id.note_title);
		title.setText(note.getTitle());
		
		TextView dateString = (TextView) view.findViewById(R.id.note_date);
		
        String dateText;
			
        Date date = note.getDate();
        Date now = new Date();

		Calendar noteCalendar = Calendar.getInstance();
		noteCalendar.setTime(date);

		Calendar nowCalendar = Calendar.getInstance();
		nowCalendar.setTime(now);

		if (sameDay(noteCalendar, nowCalendar)) {
            dateText = Utility.getTodayString(date);
        } else {
            dateText = Utility.getOldateString(date);
        }
        dateString.setText(dateText);

		Animation animation = AnimationUtils.loadAnimation(activity,
				R.anim.fadeout);
		view.clearAnimation();
		view.startAnimation(animation);

		return view;
    }

	private boolean sameDay(Calendar calendar, Calendar nowCalendar) {
		return calendar.get(Calendar.DAY_OF_YEAR) == nowCalendar.get(Calendar
				.DAY_OF_YEAR) && calendar.get(Calendar.YEAR) == nowCalendar
				.get(Calendar.YEAR);
	}

	public void removeNote(long id) {
        for (int i = 0; i < notes.size(); i++) {
            if (notes.get(i).getId() == id) {
                notes.remove(i);
                break;
            }
        }

        notifyDataSetChanged();
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position,
                            long id) {
        NoteBean item = (NoteBean) getItem(position);

        Intent intent = new Intent(activity, ShowNoteActivity.class);
        intent.putExtra(Constants.EXTRA_NOTE_ID, item.getId());
        activity.startActivity(intent);
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> parent, View view,
                                   int position, long id) {
        NoteActionsDialog dialog = new NoteActionsDialog();
        dialog.setContext(activity);
        dialog.setNote((NoteBean) getItem(position));
        dialog.setListChangedListener(new OnListChangedListener() {
            @Override
            public void onListChanged() {
                notifyDataSetChanged();
            }

            @Override
            public void onNoteDeleted(long id) {
				for (int i = 0, notesSize = notes.size(); i < notesSize; i++) {
					NoteBean note = notes.get(i);
					if (id == note.getId()) {
						notes.remove(note);
						break;
					}
				}

				super.onNoteDeleted(id);
			}
        });
        dialog.show(activity.getSupportFragmentManager(), NOTE_ACTIONS_DIALOG);

        return true;
    }

}