package ru.ttyh.neko259.notey.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Parser {

    private static final String TAG_MONOSPACE_START = "<tt>";
    private static final String TAG_MONOSPACE_END = "</tt>";
    private static final String HTML_START = "<html><body>";
    private static final String HTML_END = "</body></html>";

    private static final Map<String, String> doubleTags
            = new HashMap<String, String>();
    private static final Map<String, String> singleTags
            = new HashMap<String, String>();
    private static final Map<String, String> whitespaceTags
            = new HashMap<String, String>();

	static {
        whitespaceTags.put("\n", "<br />");

        doubleTags.put("\\[b\\](.+)\\[/b\\]", "<b>$1</b>");
        doubleTags.put("\\[i\\](.+)\\[/i\\]", "<i>$1</i>");

        singleTags.put("\\[ \\]", "<font color=\"red\">[-]</font>");
		singleTags.put("\\[\\+\\]", "<font color=\"#00FF00\">[+]</font>");
		singleTags.put("\\[\\.\\]", "<font color=\"yellow\">[…]</font>");
		singleTags.put("\\[\\?\\]", "<font color=\"#FF7700\">[?]</font>");
		singleTags.put("\\[\\*\\]", "<font color=\"#2299FF\">[*]</font>");
	}

    public static final String TAG_ESCAPES = "\\";

    public static String parseNoteText(String rawText) {
		String tempText = rawText;
		
		for (String key : doubleTags.keySet()) {
			tempText = tempText.replaceAll(key, doubleTags.get(key));
        }
        for (String key : whitespaceTags.keySet()) {
            tempText = tempText.replaceAll(key, whitespaceTags.get(key));
        }
        for (String key : singleTags.keySet()) {
			tempText = tempText.replaceAll(key, TAG_MONOSPACE_START
                    + singleTags.get(key) + TAG_MONOSPACE_END);
		}
		
		return tempText;
	}

    public static List<String> getSingleTags() {
        List<String> tags = new ArrayList<String>();
        tags.addAll(singleTags.keySet());

        return tags;
    }

    public static List<String> getDoubleTags() {
        List<String> tags = new ArrayList<String>();
        tags.addAll(doubleTags.keySet());

        return tags;
    }

    public static String getTagName(String tag) {
        return tag.replace(TAG_ESCAPES, "");
    }

    public static String getHtmlNote(String rawNote) {
        return HTML_START + parseNoteText(rawNote) + HTML_END;
    }

}