package ru.ttyh.neko259.notey.util.export;

import android.os.Environment;
import android.util.Xml;
import org.xml.sax.SAXException;
import org.xmlpull.v1.XmlSerializer;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.DBManager;
import ru.ttyh.neko259.notey.db.NoteDAO;

import java.io.*;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

public class Exporter {

    private static final String DIR_EXPORT = "NoteyExport";

    public static final String TAG_NOTE = "note";
    public static final String TAG_TITLE = "title";
    public static final String TAG_TEXT = "text";
	public static final String TAG_LATITUDE = "latitude";
	public static final String TAG_LONGITUDE = "longitude";

    private static final String DEFAULT_ENCODING = "utf-8";
    private static final String DEFAULT_NAMESPACE = "";
    public static final String XML_EXTENSION = ".xml";
    public static final String DEFAULT_TIMEZONE = "UTC";
    public static final String DATE_FORMAT = "yyyy-MM-dd HH-mm-ss";

    public static void exportNote(long noteId) throws IOException {
        XmlSerializer serializer = Xml.newSerializer();

        OutputStream os = null;
        Writer writer = null;

        try {
            NoteDAO noteDAO = DBManager.getInstance().getNoteDAO();
            NoteBean note = noteDAO.getNoteById(noteId);

            File noteFile = getNoteFile(note);

            os = new FileOutputStream(noteFile);
            writer = new OutputStreamWriter(os);

            serializer.setOutput(writer);

            serializer.startDocument(DEFAULT_ENCODING, false);
            serializer.startTag(DEFAULT_NAMESPACE, TAG_NOTE);

            writeTag(serializer, TAG_TITLE, note.getTitle());
            writeTag(serializer, TAG_TEXT, note.getText());
			writeTag(serializer, TAG_LATITUDE,
					String.valueOf(note.getLatitude()));
			writeTag(serializer, TAG_LONGITUDE,
					String.valueOf(note.getLongitude()));

            serializer.endTag(DEFAULT_NAMESPACE, TAG_NOTE);
            serializer.endDocument();
        } finally {
            if (writer != null) {
                try {
                    writer.close();
                } catch (IOException ioe) {
                    // Don't need to handle
                }
            }
        }
    }

    public static NoteBean importNote(File noteFile) throws IOException,
            SAXException, ParseException {
        Reader reader = null;
        InputStream is = null;

        ImportHandler handler = new ImportHandler();

        try {
            is = new FileInputStream(noteFile);
            reader = new InputStreamReader(is);

            Xml.parse(reader, handler);
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException ioe) {
                    // Don't need to handle
                }
            }
        }

        String fileName = noteFile.getName();
        Date date = getDateFormat().parse(fileName.substring(0, fileName.length
                () - XML_EXTENSION.length()));

        NoteBean note = handler.getNote();
        note.setDate(date);

        return note;
    }

    public static File getExportDirectory() {
        File externalStorage = Environment.getExternalStorageDirectory();
        File exportDir = new File(externalStorage, DIR_EXPORT);

        if (!exportDir.exists()) {
            exportDir.mkdirs();
        }

        return exportDir;
    }

    public static void clearExportDirectory() {
        File directory = getExportDirectory();
        for (File file : directory.listFiles()) {
            file.delete();
        }
    }

    private static void writeTag(XmlSerializer serializer, String tag,
                                 String text) throws IOException {
        serializer.startTag(DEFAULT_NAMESPACE, tag);
        serializer.text(text);
        serializer.endTag(DEFAULT_NAMESPACE, tag);
    }

    private static File getNoteFile(NoteBean note) throws IOException {
        Date noteDate = note.getDate();

        String fileName = getDateFormat().format(noteDate);

        File noteFile = new File(getExportDirectory(), fileName
                + XML_EXTENSION);
        noteFile.createNewFile();

        return noteFile;
    }

    private static DateFormat getDateFormat() {
        TimeZone tz = TimeZone.getTimeZone(DEFAULT_TIMEZONE);

        DateFormat format = new SimpleDateFormat(DATE_FORMAT);
        format.setTimeZone(tz);

        return format;
    }

}